<?php
/*
 * Gallery - a web based photo album viewer and editor
 * Copyright (C) 2000-2007 Bharat Mediratta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
 */

$VERSION = 1.1;

@ini_set('display_errors', 1);

require_once('embed.php');
GalleryEmbed::init();
$count = 0;
global $gallery;

$user = $gallery->getActiveUser();
?>

<h1> Fix Broken Filenames (version <?php print $VERSION ?>)</h1>
<p>
This tool will give you a series of move commands to fix all files
that have the right filename but have lowercase filenames.  This can
happen if you store your files on a server that isn't case sensitive
(like a Windows FAT32 drive).  This script will <b>only</b> give you a
list of commands to run, it won't take any action directly!  You need
to cut and paste the commands into a file and run them to fix the
problem.

<h2>Notes</h2>
<ul>
  <li>
    This script can use a lot of memory.  If it aborts with an "out of
    memory" error, try raising the memory limit in your php.ini file
  </li>
  <li>
    This script is probably pretty slow for large Gallery installs.  Just
    be patient!
  </li>
  <li>
    You can run this script as many times as you want.  If you
    see nothing between the two horizontal lines below, then there are no files that need fixing.
  </li>
  <li>
    If you want to check every image in your Gallery, you should make sure you're
    <a href="main.php?g2_view=core.UserLogin">logged in</a> as a Site Administrator.<br/>
    (You're logged in as user: <b><?php print $user->getUsername(); ?></b>)
  </li>
</ul>

<?php if (!isset($_REQUEST['go'])): ?>
<form>
  <input type="button" value="Go!" onclick="this.value='Thinking.. be patient!'; this.disabled='disabled'; location.href = '?go=1';">
</form>
<?php endif; ?>
</p>

<hr>
<pre>
<?php
if (isset($_REQUEST['go'])) {
    list ($ret, $defaultAlbumId) = GalleryCoreApi::getPluginParameter('module', 'core', 'id.rootAlbum');
    if ($ret) {
	print $ret->getAsHtml();
    } else {
	$ret = walk($defaultAlbumId);
    }
    if ($ret) {
	print $ret->getAsHtml();
    }
    print "<hr>$count items checked successfully<br>";
}

function walk($id) {
    global $count;

    $ids = array($id);
    $entities = array();

    $ret = loadEntitiesIntoTable($ids, $entities);
    if ($ret) {
	return $ret;
    }

    while (!empty($ids)) {
        $count++;
	$id = array_pop($ids);

	$entity = $entities[$id];
	unset($entities[$id]);

	if (GalleryUtilities::isA($entity, 'GalleryAlbumItem')) {
	    list ($ret, $newIds) = GalleryCoreApi::fetchChildItemIds($entity);
	    if ($ret) {
		return $ret;
	    }

	    if (!empty($newIds)) {
		$ret = loadEntitiesIntoTable($newIds, $entities);
		if ($ret) {
		    return $ret;
		}

		$ids = array_merge($ids, $newIds);
	    }
	}


	if (GalleryUtilities::isA($entity, 'GalleryDataItem')) {
	    list ($ret, $path) = $entity->fetchPath();
	    if ($ret) {
		return $ret;
	    }

	    if (!file_exists($path)) {
		$dirname = dirname($path);
		$basename = basename($path);
		$lcase = strtolower($basename);

		if (file_exists("$dirname/$lcase")) {
		    print "mv $dirname/$lcase $path<br>";
		}
	    }
	}
    }
}

function loadEntitiesIntoTable($ids, &$table) {
    list ($ret, $entities) = GalleryCoreApi::loadEntitiesById($ids);
    if ($ret) {
	return $ret;
    }
    foreach ($ids as $id) {
	$table[$id] = array_shift($entities);
    }

    return null;
}
?>
